@@ -1,9 +1,11 @@
 import sys
 import os
 import json
+import jsonlines as jsonl
 from .client import Client
 from .subtypes import Token, Subsentence
 from .utils import flatten_lst, StrProperty, ListProperty, DictProperty
+from .TextChunk import TextChunk
 
 #Accepted arguments for hierarchical levels
 GLOBAL =    ['g', 'global', 'glob']
@@ -15,13 +17,41 @@ TOK =       ['t', 'token', 'tok', 'tokens']
 class PipelineError(Exception): pass
 class RequestError(Exception): pass
 
-class Sentence:
-    __slots__ = ("data", "n", "max")
+def clear_data(data_json):
+    def clean_recursif(node):
+        if isinstance(node, list):
+            for e in node:
+                clean_recursif(e)  
+        elif isinstance(node, dict):
+            keys = list(node.keys())
+            for k in keys:
+                if k == 'confidence' or node[k] in [[], {}, None]:
+                    node.pop(k)
+                elif isinstance(node[k], dict):
+                    clean_recursif(node[k])
+                    if not node[k]:
+                        node.pop(k)
+                elif isinstance(node[k], list):
+                    clean_recursif(node[k])
+                    if not node[k]:
+                        node.pop(k)
+        else:
+            return None
 
-    def __init__(self, data_sentence):
-        self.data = data_sentence
+    data_json = {k:v for k,v in data_json.items() if v and k in ['source', 'language_used', 'source_pure', 'ml_sentiment', 'proposition', 'sentiment', 'sentence_acts', 'ml_emotion', 'emotion', 'synthesis']}
+    data_json['synthesis'] = [{k:v for k,v in i.items() if v not in [[], {}, None]} for i in data_json.get('synthesis', [])]
+    clean_recursif(data_json)
+    return data_json
+
+class Sentence(TextChunk):
+    __slots__ = ("data", "n", "max", "id")
+
+    def __init__(self, data_sentence, idx=0):
+        super(Sentence, self).__init__()
+        self.data = clear_data(data_sentence)
         self.max = len(self.data.get('synthesis', []))
         self._ner_fix()
+        self.id = idx
 
     # To modify when desambiguisation is active and only one NER entity is returned for each token
     def _ner_fix(self):
@@ -37,10 +67,11 @@ class Sentence:
                 if 'sub' in m and m['sub']:
                     if m['sub'] == 'number':
                         ner.append(m['sub'])
-            self.data['synthesis'][i]['type'] = ner
+            if ner:
+                self.data['synthesis'][i]['type'] = ner
 
     def __repr__(self):
-        return str(self.data)
+        return self.str
 
     def __iter__(self):
         self.n = 0
@@ -49,32 +80,42 @@ class Sentence:
     def __next__(self):
         if self.n < self.max:
             self.n += 1
-            return Token(self.data.get('synthesis', [])[self.n - 1])
+            return Token(self.data.get('synthesis', [])[self.n - 1], self.n -1, self.data.get('source_pure', self.data.get('source', None)))
         else:
             raise StopIteration
 
-    def _get_subsentence(self, id, idx):
+    @property
+    def idx(self):
+        return self.id
+
+    @ListProperty
+    def sentences(self):
+        return [self]
+
+    def _get_subsentence(self, _id, _idx):
         ''' Dividing information for each subsentence, then it can use 
         normal Sentence methods to access its data'''
         data = {}
-        data['synthesis'] = self.data['synthesis'][idx['start_id']:idx['end_id'] + 1]
-        if len(self.data['sentiment'].get('subsentences', [])) > id:
-            data['sentiment'] = self.data['sentiment'].get('subsentences', [])[id]
-        if len(self.data.get('ml_emotion', {}).get('subsentence', [])) > id:
-            data['ml_emotion'] = {'sentence': self.data.get('ml_emotion', {}).get('subsentence', [])[id]}
-        if len(self.data.get('ml_sentiment', {}).get('subsentence', [])) > id:
-            data['ml_sentiment'] = {'sentence': self.data['ml_sentiment'].get('subsentence', [])[id]}
+        data['synthesis'] = self.data['synthesis'][_idx['start_id']:_idx['end_id'] + 1]
+        data['source'] = ' '.join([k.get('source', '') if k.get('source', '') else '' for k in data['synthesis'][_idx['start_id']:_idx['end_id'] + 1]])
+        data['source_pure'] = self.data.get('source_pure', '')
+        if len(self.data.get('sentiment', {}).get('subsentences', [])) > _id:
+            data['sentiment'] = self.data['sentiment'].get('subsentences', [])[_id]
+        if len(self.data.get('ml_emotion', {}).get('subsentence', [])) > _id:
+            data['ml_emotion'] = {'sentence': self.data.get('ml_emotion', {}).get('subsentence', [])[_id]}
+        if len(self.data.get('ml_sentiment', {}).get('subsentence', [])) > _id:
+            data['ml_sentiment'] = {'sentence': self.data['ml_sentiment'].get('subsentence', [])[_id]}
         return Subsentence(data)
 
     @ListProperty
     def tokens(self):
-        return [Token(s) for s in self.data.get('synthesis', [])]
+        return [Token(s, i, self.data.get('source_pure', self.data.get('source', None))) for i, s in enumerate(self.data.get('synthesis', []))]
 
     @ListProperty
     def subsentences(self):
         return [self._get_subsentence(id, idx) for id, idx in enumerate(self.data.get('proposition', []))]
 
-    @property
+    @ListProperty
     def token(self):
         return [s.get('source', None) for s in self.data.get('synthesis', [])]
 
@@ -82,10 +123,18 @@ class Sentence:
     def str(self):
         return self.data.get('source', None)
 
+    @StrProperty
+    def original_text(self):
+        return self.data.get('source_pure', self.data.get('source', None))
+
     @ListProperty
     def lemma(self):
         return [s.get('lemma', None) for s in self.data.get('synthesis', [])]
 
+    @ListProperty
+    def lemma_detail(self):
+        return flatten_lst([s.lemma_detail for s in self.tokens])
+
     @ListProperty
     def synthesis(self):
         return self.data.get('synthesis', [])
@@ -94,6 +143,10 @@ class Sentence:
     def pos(self):
         return [s.get('tag', None) for s in self.data['synthesis']]
 
+    @ListProperty
+    def pos_detail(self):
+        return flatten_lst([s.pos_detail for s in self.tokens])
+
     @ListProperty
     def dep(self):
         return [s.get('dep', None) for s in self.data['synthesis']]
@@ -104,15 +157,15 @@ class Sentence:
 
     @ListProperty
     def meaning(self):
-        return [[(m['super'] or '', m['sub'] or '') for m in t.get('meaning', [])] for t in self.data['synthesis']]
+        return [[(m.get('super', ''), m.get('sub', '')) for m in t.get('meaning', [])] for t in self.data['synthesis']]
 
     @ListProperty
     def emotion(self):
-        return [(e.get('type', None), round(e.get('value', 0), 4)) for e in self.data.get('ml_emotion', {}).get('sentence', [])]
+        return [(k, round(v,4)) for k,v in self.data.get('emotion', {}).get('values', {}).items() if v != 0]
 
     @DictProperty
     def emotion_ml(self):
-        return self.emotion
+        return [(e.get('type', None), round(e.get('value', 0), 4)) for e in self.data.get('ml_emotion', {}).get('sentence', [])]
 
     @DictProperty
     def sentiment_ml(self):
@@ -142,17 +195,18 @@ class Sentence:
     def coreference(self):
         return [t.coreference for t in self.tokens]
 
-class Document:
+class Document(TextChunk):
     __slots__ = ("sentences", "data", "n", "max", "id")
-    next_id = 0
-    def __init__(self, sentences, id=None):
-        self.sentences = [Sentence(s) for s in sentences]
+    def __init__(self, sentences, _id):
+        super(Document, self).__init__()
+        self.sentences = [Sentence(s, i) for i, s in enumerate(sentences)]
         self.max = len(self.sentences)
         self.data = self.sentences
-        if id is None:
-            id = Document.next_id
-            Document.next_id += 1
-        self.id = str(id)
+        self.id = str(_id)
+
+    @property
+    def idx(self):
+        return self.id
 
     @ListProperty
     def subsentences(self):
@@ -162,6 +216,10 @@ class Document:
     def tokens(self):
         return flatten_lst([s.tokens for s in self.sentences])
 
+    @ListProperty
+    def documents(self):
+        return [self]
+
     def add_sentence(self, sentence):
         self.sentences.append(sentence)
         self.max += 1
@@ -183,13 +241,14 @@ class Document:
         else:
             raise StopIteration
 
-class NLP:
+class NLP(TextChunk):
     """ Class for data analysis of API return.
         Takes a Client class as input which is used to make api requests.
         Provides both high and low level methods to access data via specific class
         for each key in the api result or methods designed for specific use cases.
         """
     def __init__(self, api_key = None, client = None, data = None, no_print=False):
+        super(NLP, self).__init__()
         self.client = None
         if client or api_key:
             self.add_client(client, api_key)
@@ -197,6 +256,7 @@ class NLP:
         self.max = len(self.documents)
         self.data = self.documents
         self.fields = [p for p in dir(Sentence) if isinstance(getattr(Sentence,p),property)]
+        self._next_id = 0
         if 'token_flat' not in self.fields:
             self._generate_properties()
 
@@ -206,12 +266,6 @@ class NLP:
         if no_print:
             print = doNothing
 
-    def statistics(self):
-        return {'documents': len(self.documents),
-        'sentences': len(self.sentences),
-        'subsentences': len(self.subsentences),
-        'tokens': len(self.tokens)}
-
     @ListProperty
     def sentences(self):
         """ Direct access to all sentences"""
@@ -228,111 +282,27 @@ class NLP:
         """ Direct access to all tokens"""
         return flatten_lst([s.tokens for s in self.sentences])
 
-    def vocabulary(self, filter_pos = None, lemma=False, level='global'):
-        """ Generates vocabulary list of words, document or global level """
-        if level not in DOC + GLOBAL:
-            print('Only global or document level available.')
-            return []
-        vocabulary = []
-        if level in GLOBAL:
-            tokens = self.token_flat if not lemma else self.lemma_flat
-            pos = self.pos_flat
-            for t, p in zip(tokens, pos):
-                if (t,p) not in vocabulary:
-                    if not filter_pos or p in filter_pos:
-                        vocabulary.append((t, p))                    
-        elif level in DOC:
-            for doc in self.documents:
-                tokens = doc.token_flat if not lemma else doc.lemma_flat
-                pos = doc.pos_flat
-                doc_vocabulary = []
-                for t, p in zip(tokens, pos):
-                    if (t,p) not in doc_vocabulary:
-                        if not filter_pos or p in filter_pos:
-                            doc_vocabulary.append((t, p))      
-                vocabulary.append(doc_vocabulary)
-        return vocabulary
-
-    def word_count(self, filter_pos = None, lemma=False, level='global'):
-        """ Generates word count, document or global level """
-        if level not in DOC + GLOBAL:
-            print('Only global or document level available.')
-            return []
-        word_count = []
-        if level in GLOBAL:
-            tokens = self.token_flat if not lemma else self.lemma_flat
-            pos = self.pos_flat
-            word_count = {}
-            for t, p in zip(tokens, pos):
-                if not filter_pos or p in filter_pos:
-                    word_count[(t, p)] = word_count.get((t,p), 0) + 1
-        elif level in DOC:
-            for doc in self.documents:
-                tokens = doc.token_flat if not lemma else doc.lemma_flat
-                pos = doc.pos_flat
-                doc_word_count = {}
-                for t, p in zip(tokens, pos):
-                    if not filter_pos or p in filter_pos:
-                        doc_word_count[(t, p)] = doc_word_count.get((t,p), 0) + 1
-                word_count.append(doc_word_count)
-        return word_count
-
-    def word_frequency(self, filter_pos = None, lemma=False, level='global'):
-        """ Generates frequency list of words """
-        if level not in DOC + GLOBAL:
-            print('Only global or document level available.')
-            return None
-        vocab = self.word_count(filter_pos=filter_pos, lemma=lemma, level=level)
-        if level in GLOBAL:
-            total = sum(self.word_count().values())
-            return {k:round(v/(total + 1e-10), 10) for k,v in vocab.items()}
-        elif level in DOC:
-            total = [sum(v.values()) for v in self.word_count( level=level)]
-            return [{k:round(val/(t + 1e-10), 10) for k,val in v.items()} for v, t in zip(vocab, total)]
-
-    def list_entities(self, level='global'):
-        """ Returns dictionary (or list of dict) of ner entities at the specified level"""
-        if level in GLOBAL:
-            _iter = [self]
-        elif level in DOC:
-            _iter = self.documents
-        elif level in SENT:
-            _iter = self.sentences
-        elif level in SUB:
-            _iter = self.subsentences
-        else:
-            print("'" + str(level) + "' is an incorrect value for level argument.")
-            return []
-        entities = []
-        for d in _iter:
-            tmp = {}
-            for t, e in zip(d.token_flat, d.ner_flat):
-                if e.get('type', None):
-                    for type in e['type']:
-                        if type in tmp:
-                            tmp[type].append(t)
-                        else:
-                            tmp[type] = [t]
-            entities.append(tmp)
-        return entities
+    @property
+    def nlp(self):
+        return [self]
 
     def _generate_properties(self):
         """ Takes properties of the Sentence class and dynamically create properties for 
             NLP, Document and Subsentence class."""
         for _class in [Document, NLP]: #adding to nlp and document
             for field in self.fields:
-                if field in ['subsentences', 'tokens']:
+                if field in ['subsentences', 'tokens', 'sentences']:
                     continue
                 setattr(_class, field, property(self._make_lambda(field)))
                 setattr(_class, field + '_flat', property(self._make_lambda(field, True)))
         for _class in [Subsentence]: #adding specific properties for subsentence
             for field in [p for p in dir(Sentence) if isinstance(getattr(Sentence, p), property)]:
                 filters = ['sentiment_target', 'morphology', 'ner', 'token', 'str', \
-                            'lemma', 'synthesis', 'pos', 'dep', 'language', 'meaning', \
+                            'lemma', 'lemma_detail', 'synthesis', 'pos', 'pos_detail', 'dep', 'language', 'meaning', \
                             'sentiment', 'sentiment_ml', 'emotion', 'emotion_ml', 'tokens']
                 if field in filters:
                     setattr(_class, field, getattr(Sentence, field))
-                else:
+                elif field not in ['subsentences']:
                     setattr(_class, field, property(self._make_lambda_sub(field)))
         for _class in [Sentence, Subsentence]: #adding flat variants to sentence and subsentence for convenience
             for field in self.fields:
@@ -368,7 +338,7 @@ class NLP:
         """ Check if synthesis key is empty """
         bad_idx = []
         for i, r in enumerate(result):
-            if not r['synthesis']:
+            if not 'synthesis' in r:
                 bad_idx.append(i)
         return bad_idx
 
@@ -411,7 +381,7 @@ class NLP:
         else:
             print('Please provide a client instance or an api_key.')
 
-    def add_document(self, document, skip_document=False, id=None):
+    def add_document(self, document, skip_document=False, id=None, verbose=True):
         """ Performs request to lettria API for a document and stores it.
             skip_document: If False document is added even if empty or request failed.
             id: Id given to document, by default sequential integer """
@@ -445,15 +415,19 @@ class NLP:
             else:
                 print("Skpping document, processing failed.")
         else:
+            if id == None:
+                id = self._next_id
+                self._next_id += 1
             if isinstance(results, list):
-                self.documents.append(Document(results, id=id))
-                print("Added document " + str(self.documents[-1].id) + '.')
+                self.documents.append(Document(results, _id=id))
+                if verbose:
+                    print("Added document " + str(self.documents[-1].id) + '.')
             else:
                 if not document:
-                    self.documents.append(Document([], id=id))
+                    self.documents.append(Document([], _id=id))
                     print("Added empty document " + str(self.documents[-1].id) + ': received empty input.')
                 else:
-                    self.documents.append(Document([], id=id))
+                    self.documents.append(Document([], _id=id))
                     print("Added empty document " + str(self.documents[-1].id) + ': processing failed.')
             self.max += 1
 
@@ -478,20 +452,22 @@ class NLP:
         if not file:
             file = 'results'
             while not path_ok:
-                path = file + '_' + str(c) + '.json'
+                path = file + '_' + str(c) + '.jsonl'
                 if not os.path.isfile(path):
                     path_ok = 1
                 else:
                     c += 1
         else:
             path = file
-            if path.endswith('.json'):
-                path = path[:-5]
-            path = path + '.json'
+            if path.endswith('.jsonl'):
+                path = path[:-6]
+            path = path + '.jsonl'
         try:
-            with open(path, 'w') as f:
-                json.dump({'document_ids': [d.id for d in self.documents],'documents':self._get_data()}, f)
-            print(f'Results saved to {path}\n')
+            with jsonl.open(path, 'w') as fw:
+                for d in self.documents:
+                    fw.write({'document_id':d.id, 'data':d._get_data()})
+                # json.dump({'document_ids': [d.id for d in self.documents],'documents':self._get_data()}, f)
+            print(f'Results saved to {path}')
         except Exception as e:
             print(e)
 
@@ -499,30 +475,59 @@ class NLP:
         """ Alias for load_results"""
         self.load_results(*args)
 
-    def load_results(self, path = 'results_0', reset = False):
+    def load_results(self, path = 'results_0', reset = False, chunksize = None):
         """ Loads result from a valid json file."""
-        if path.endswith('.json'):
-            path = path[:-5]
-        path = path + '.json'
+        if path.endswith('.json') or path.endswith('.jsonl'):
+            pass
+        else:
+            path = path + '.jsonl'
         try:
-            with open(path, 'r') as f:
-                result = json.load(f)
-                if reset:
-                    self.reset_data()
-                if isinstance(result, dict):
-                    for id_, r in zip(result['document_ids'], result['documents']):
-                        self.add_document_data(r, id=id_)
-                else:
-                    for r in result:
-                        self.add_document_data(r)
-            print(f'Loaded {path} successfully\n')
+            if reset:
+                self.reset_data()
+            if path.endswith('jsonl'):
+                with jsonl.open(path, 'r') as f:
+                    for line in f:
+                        self.add_document_data(line.get('data'), id=line.get('document_id', None))
+            elif path.endswith('json'):
+                with open(path, 'r') as f:
+                    result = json.load(f)
+                    if isinstance(result, dict):
+                        assert len(result.get('document_ids', [])) == len(result.get('documents', [])), \
+                                "'document_ids' and 'documents' should be of similar length"
+                        for id_, r in zip(result['document_ids'], result['documents']):
+                            self.add_document_data(r, id=id_)
+                    else:
+                        for r in result:
+                            self.add_document_data(r)
+            print(f'Loaded {path} successfully')
         except Exception as e:
             print('Failure to load ' + str(path) + ': ')
             print(e, '\n')
 
+    def split_results(self, input_file, max_document_per_file, output_file=''):
+        """ Splits existing json results into multiple files """
+        if not output_file:
+            output_file = input_file
+        with open(input_file, 'r') as f:
+            results = json.load(f)
+        i = 0
+        count = 0
+        while i < len(self.documents):
+            if isinstance(results, list):
+                with open(f"{count}_{output_file}", 'w') as fw:
+                    json.dump(result[i:i + max_document_per_file], fw)
+            elif isinstance(results, dict):
+                with open(f"{count}_{output_file}", 'w') as fw:
+                    json.dump({'document_ids':results['document_ids'][i:i + max_document_per_file],\
+                             'documents':results['documents'][i:i + max_document_per_file]}, fw)
+            i += max_document_per_file
+            count += 1
+            results['document_ids']
+        # for n in number:
+
     def reset_data(self):
         """ Erase current data """
         self.documents = []
         self.data = self.documents
         self.max = 0
-        Document.next_id = 0
\ No newline at end of file
+        self._next_id = 0
\ No newline at end of file
