@@ -10,7 +10,7 @@ class Client:
 			key = key[14:]
 		self.key = key
 		self.headers = { 'Authorization': 'LettriaProKey ' + str(self.key), 'content-type': 'application/json' }
-		self.max_try = 5
+		self.max_try = 3
 
 	def print_response_error(self, response):
 		if 'Error' in response:
@@ -24,7 +24,7 @@ class Client:
 		i = 0
 		while i < self.max_try:
 			try:
-				response = requests.post('http://51.254.207.74:4300/api/main', headers=self.headers, json={'text' : text}).json()
+				response = requests.post('https://api.lettria.com/main', headers=self.headers, json={'text' : text}).json()
 				if response and not isinstance(response, list):
 					raise Exception
 				result = response
