@@ -1,3 +1,6 @@
+import os
+
+debug = os.getenv('DEBUG') == '1' or os.getenv('DEBUG') == 'True'
 flatten=lambda l: sum(map(flatten,l),[]) if isinstance(l,list) else [l]
 
 def flatten_lst(lst):
@@ -11,8 +14,11 @@ class StrProperty(property):
             if self.fget is None:
                 raise AttributeError("unreadable attribute")
             return self.fget(obj)
-        except:
-            return None
+        except Exception as e:
+            if debug:
+                raise e
+            else:
+                return None
 
 class ListProperty(property):
     def __get__(self, obj, objtype=None):
@@ -22,8 +28,11 @@ class ListProperty(property):
             if self.fget is None:
                 raise AttributeError("unreadable attribute")
             return self.fget(obj)
-        except:
-            return []
+        except Exception as e:
+            if debug:
+                raise e
+            else:
+                return []
 
 class DictProperty(property):
     def __get__(self, obj, objtype=None):
@@ -33,5 +42,8 @@ class DictProperty(property):
             if self.fget is None:
                 raise AttributeError("unreadable attribute")
             return self.fget(obj)
-        except:
-            return {}
\ No newline at end of file
+        except Exception as e:
+            if debug:
+                raise e
+            else:
+                return {}
\ No newline at end of file
